from stretch_body.end_of_arm import EndOfArm


class ToolUSBCamWrist(EndOfArm):
    def __init__(self, name='tool_usbcam_wrist'):
        EndOfArm.__init__(self, name)

    def take_picture(self):
        raise NotImplementedError()

    def return_stream(self):
        raise NotImplementedError()



#!/usr/bin/env python
from stretch_body.hello_utils import *
import sys
import stretch_body.wrist_yaw as wrist_yaw
import stretch_tool_share.reactorx_wrist_v1.wrist_pitch as wrist_pitch
import stretch_tool_share.reactorx_wrist_v1.wrist_roll as wrist_roll
import stretch_tool_share.reactorx_wrist_v1.reactor_gripper as reactor_gripper
import argparse
print_stretch_re_use()

parser=argparse.ArgumentParser(description='Jog the Reactor Wrist wrist joints from the keyboard')
group = parser.add_mutually_exclusive_group(required=True)
group.add_argument("--yaw", help="Jog yaw joint",action="store_true")
group.add_argument("--pitch", help="Jog pitch joint",action="store_true")
group.add_argument("--roll", help="Jog roll joint",action="store_true")
group.add_argument("--gripper", help="Jog gripper joint",action="store_true")

args, _ = parser.parse_known_args()



w=None

if args.yaw:
    print("Controlling Yaw Joint")
    w=wrist_yaw.WristYaw()
    w.startup()
    poses = {'zero': 0, 'X': deg_to_rad(90), 'Y': deg_to_rad(-45)}
elif args.pitch:
    print("Controlling Pitch Joint")
    w=wrist_pitch.WristPitch()
    w.startup()
    poses = {'zero': 0, 'X': deg_to_rad(55), 'Y': deg_to_rad(-90)}
elif args.roll:
    print("Controlling Roll Joint")
    w = wrist_roll.WristRoll()
    w.startup()
    poses = {'zero': 0, 'X': deg_to_rad(90), 'Y': deg_to_rad(-90)}
elif args.gripper:
    print("Controlling Gripper Joint")
    w = reactor_gripper.ReactorGripper()
    w.startup()
    poses = {'zero': 0, 'X': deg_to_rad(90), 'Y': deg_to_rad(-90)}

if w is not None:
    v_des=w.params['motion']['default']['vel']
    a_des=w.params['motion']['default']['accel']

    def menu_top():
        print('------ MENU -------')
        print('m: menu')
        print('a: increment 15 deg')
        print('b: decrement 15 deg')
        print('p: position (ticks)')
        print('x: pose X')
        print('y: pose Y')
        print('z: pose zero')
        print('1: speed slow')
        print('2: speed default')
        print('3: speed fast')
        print('4: speed max')
        print('-------------------')

    def step_interaction():
        global v_des, a_des
        menu_top()
        x=sys.stdin.readline()
        if len(x)>1:
            if x[0]=='m':
                menu_top()
            if x[0]=='p':
                p = w.ticks_to_world_rad(float(x[1:]))
                w.move_to(p,v_des,a_des)

            if x[0] == 'a':
                w.move_by(deg_to_rad(15), v_des, a_des)

            if x[0] == 'b':
                w.move_by(deg_to_rad(-15), v_des, a_des)

            if x[0] == '1':
                v_des = w.params['motion']['slow']['vel']
                a_des = w.params['motion']['slow']['accel']

            if x[0] == '2':
                v_des = w.params['motion']['default']['vel']
                a_des = w.params['motion']['default']['accel']

            if x[0] == '3':
                v_des = w.params['motion']['fast']['vel']
                a_des = w.params['motion']['fast']['accel']

            if x[0] == '4':
                v_des = w.params['motion']['max']['vel']
                a_des = w.params['motion']['max']['accel']

            if x[0] == 'x':
                w.move_to(poses['X'], v_des, a_des)
            if x[0] == 'y':
                w.move_to(poses['Y'], v_des, a_des)
            if x[0] == 'z':
                w.move_to(poses['zero'], v_des, a_des)
        else:
            w.pretty_print()

    try:
        while True:
            try:
                step_interaction()
            except (ValueError):
                print('Bad input...')
            w.pull_status()
    except (ThreadServiceExit, KeyboardInterrupt):
        w.stop()




#! /usr/bin/env python

from stretch_body.robot_collision import *
from stretch_body.hello_utils import *
import math


class CollisionStretchDexWristToSelf(RobotCollisionModel):
    """
    NOTE: Experimental. You may want to turn this off in the params (enable=0)
    Manage collisions of the standard Stretch Gripper tool with the
    ground and the base
    """

    def __init__(self, collision_manager):
        RobotCollisionModel.__init__(self, collision_manager, 'collision_stretch_dex_wrist_to_self')

    def step(self, status):
        x_roll=status['end_of_arm']['wrist_roll']['pos']
        x_pitch = status['end_of_arm']['wrist_pitch']['pos']
        x_yaw = status['end_of_arm']['wrist_yaw']['pos']

        if x_pitch>self.params['pitch_up_thresh']:
            if x_roll<=deg_to_rad(-135) or x_roll>=deg_to_rad(135): #Servo body up
                workspace='limit_pitch_up_palm_up'
            if (x_roll<=deg_to_rad(135) and x_roll>=deg_to_rad(45)) or (x_roll>=deg_to_rad(-135) and x_roll<=deg_to_rad(-45)): #Servo body sideqays
                workspace='limit_pitch_up_palm_side'
            if x_roll<=deg_to_rad(45) and x_roll>=deg_to_rad(-45): #Servo body down
                workspace='limit_pitch_up_palm_down'
            w={ 'wrist_yaw':self.params[workspace]['yaw'],'wrist_pitch':self.params[workspace]['pitch'],'wrist_roll':[None, None]}
            return w
        else:
            return  {'wrist_yaw':[None,None],'wrist_pitch':[None,None],'wrist_roll':[None, None]}

class CollisionStretchDexWristToBase(RobotCollisionModel):
    """
    NOTE: Experimental. You may want to turn this off in the params (enable=0)
    Manage collisions of the standard Stretch Gripper tool with the
    ground and the base
    """

    def __init__(self, collision_manager):
        RobotCollisionModel.__init__(self, collision_manager, 'collision_stretch_dex_wrist_to_base')
        self.fk=EndOfArmForwardKinematics()

    def step(self, status):
        cfg = {
            'joint_wrist_yaw': status['end_of_arm']['wrist_yaw']['pos'],
            'joint_wrist_pitch': status['end_of_arm']['wrist_pitch']['pos'],
            'joint_wrist_roll': status['end_of_arm']['wrist_roll']['pos'],
            'joint_gripper_finger_left':0.0
        }
        pose = self.fk.tool_fk(cfg, 'link_gripper_fingertip_left')
        forward = pose[0][3]  # Forward, pos is towards wheels
        height = pose[1][3]  # Height , pos is up, pad out
        extension = pose[2][3]  # Extension, pos is reach
        if 0:
            print('------')
            print('Forward', forward)
            print('Height' ,height)
            print('Extension', extension)
            print('Lift',status['lift']['pos'])
            print('Arm', status['arm']['pos'])
            print("Wrist pitch",rad_to_deg(status['end_of_arm']['wrist_pitch']['pos']))


        wrist_yaw_fwd = deg_to_rad(90.0) #tool pointing to the front
        tool_clear_of_base = (status['end_of_arm']['wrist_yaw']['pos'] < wrist_yaw_fwd) and (status['arm']['pos'] > self.params['arm_clear_base'])
        tool_to_floor=max(0, status['lift']['pos']+self.params['base_to_floor_offset']+height)


        if tool_clear_of_base:
            lift_lower_limit = min(max(0,status['lift']['pos']-tool_to_floor), status['lift']['pos'])  # Only low it to reduce the ROM
        else:
            lift_lower_limit = min(max(self.params['palm_height'], -1*height), status['lift']['pos'])  # Only low it to reduce the ROM

        pitch_lower_limit = None
        if status['lift']['pos']>self.params['palm_height'] and status['lift']['pos']<self.params['lift_up']:
            if status['arm']['pos']<self.params['arm_clear_base']:
                q = -1*math.atan2(status['lift']['pos'],self.params['arm_clear_base'])  # How far pitch can fold back before hitting base
                pitch_lower_limit= min(status['end_of_arm']['wrist_pitch']['pos'],q)  # Only allow pitch to fold back further as lift raises, when lowering limit lift descent

        return {'lift': [lift_lower_limit, None], 'arm': [None, None], 'wrist_yaw': [None, None], 'wrist_pitch': [pitch_lower_limit, None], 'wrist_roll': [None, None]}







params = {
    "eoa_wrist_dw3_tool_tablet_12in": {
        'py_class_name': 'EOA_Wrist_DW3_Tool_Tablet_12in',
        'py_module_name': 'stretch_tool_share_se3.eoa_wrist_dw3_tool_tablet_12in.tool',
        'use_group_sync_read': 1,
        'retry_on_comm_failure': 1,
        'baud': 115200,
        'dxl_latency_timer': 64,
        'stow': {
            'arm': 0.0,
            'lift': 0.3,
            'wrist_pitch': -0.52,
            'wrist_roll': 0.0,
            'wrist_yaw': 3.0
        },
        'collision_mgmt': {
            'k_brake_distance': {'wrist_pitch': 0.25, 'wrist_yaw': 0.25, 'wrist_roll': 0.25},
            'collision_pairs': {
                'link_wrist_pitch_TO_base_link': {'link_pts': 'link_wrist_pitch', 'link_cube': 'base_link',
                                                  'detect_as': 'pts'},
                'link_wrist_yaw_bottom_TO_base_link': {'link_pts': 'link_wrist_yaw_bottom', 'link_cube': 'base_link',
                                                       'detect_as': 'pts'}},
            'joints': {'lift': [{'motion_dir': 'neg', 'collision_pair': 'link_wrist_pitch_TO_base_link'},
                                {'motion_dir': 'neg', 'collision_pair': 'link_wrist_yaw_bottom_TO_base_link'}],
                       'wrist_pitch': [],
                       'wrist_roll': [],
                       'wrist_yaw': []}},

        'devices': {
            'wrist_pitch': {
                'py_class_name': 'WristPitch',
                'py_module_name': 'stretch_body.wrist_pitch'
            },
            'wrist_roll': {
                'py_class_name': 'WristRoll',
                'py_module_name': 'stretch_body.wrist_roll'
            },
            'wrist_yaw': {
                'py_class_name': 'WristYaw',
                'py_module_name': 'stretch_body.wrist_yaw'
            }
        }}
}




params = {
    "tool_reactor_wrist": {
        'py_class_name': 'ToolReactorWrist',
        'py_module_name': 'stretch_tool_share.reactorx_wrist_v1.tool',
        'use_group_sync_read': 1,
        'retry_on_comm_failure': 1,
        'baud':115200,
        'dxl_latency_timer': 64,
        'stow': {
            'arm': 0.0,
            'lift': 0.3,
            'reactor_gripper': 0.0,
            'wrist_pitch': 2.0,
            'wrist_roll': 0.0,
            'wrist_yaw': 3.0
        },
        'devices': {
            'reactor_gripper': {
                'py_class_name': 'ReactorGripper',
                'py_module_name': 'stretch_tool_share.reactorx_wrist_v1.reactor_gripper',
            },
            'wrist_pitch': {
                'py_class_name': 'WristPitch',
                'py_module_name': 'stretch_tool_share.reactorx_wrist_v1.wrist_pitch',
                'ros_py_class_name': 'WristPitchCommandGroup',
                'ros_py_module_name': 'stretch_tool_share.reactorx_wrist_v1.command_groups'
            },
            'wrist_roll': {
                'py_class_name': 'WristRoll',
                'py_module_name': 'stretch_tool_share.reactorx_wrist_v1.wrist_roll',
                'ros_py_class_name': 'WristRollCommandGroup',
                'ros_py_module_name': 'stretch_tool_share.reactorx_wrist_v1.command_groups'
            },
            'wrist_yaw': {
                'py_class_name': 'WristYaw',
                'py_module_name': 'stretch_body.wrist_yaw',
            },
        },
        'collision_models': []
    },

    "reactor_gripper": {
        'flip_encoder_polarity': 0,
        'enable_runstop': 1,
        'gr': 1.0,
        'id': 16,
        'max_voltage_limit': 15,
        'min_voltage_limit': 11,
        'motion': {
            'default': {'accel': 8.0, 'vel': 3.0},
            'fast': {'accel': 15.0, 'vel': 6.0},
            'max': {'accel': 15.0, 'vel': 6.0},
            'slow': {'accel': 4.0, 'vel': 1.0},
            'trajectory_max': {'accel_r': 16.0, 'vel_r': 8.0},
        },
        'pid': [640, 0, 0],
        'pwm_homing': [0, 0],
        'pwm_limit': 885,
        #'range_t': [0, 4096],
        #'range_t': [1565, 3089],
	'range_t' : [0, 1150],
        'req_calibration': 0,
        'return_delay_time': 0,
        'stall_backoff': 0.017,
        'stall_max_effort': 10.0,
        'stall_max_time': 1.0,
        'stall_min_vel': 0.1,
        'temperature_limit': 72,
        'usb_name': '/dev/hello-dynamixel-wrist',
        'use_multiturn': 0,
        'zero_t': 0,
        'baud': 115200,
        'retry_on_comm_failure': 1,
        'disable_torque_on_stop': 1
    },
    "wrist_pitch": {
        'flip_encoder_polarity': 0,
        'enable_runstop': 1,
        'gr': 1.0,
        'id': 14,
        'max_voltage_limit': 15,
        'min_voltage_limit': 11,
        'motion': {
            'default': {'accel': 8.0, 'vel': 3.0},
            'fast': {'accel': 15.0, 'vel': 6.0},
            'max': {'accel': 15.0, 'vel': 6.0},
            'slow': {'accel': 4.0, 'vel': 1.0},
            'trajectory_max': {'accel_r': 16.0, 'vel_r': 8.0},
        },
        'pid': [640, 0, 0],
        'pwm_homing': [0, 0],
        'pwm_limit': 885,
        #'range_t': [650, 2048],
        'range_t': [2052, 3232],
        'req_calibration': 0,
        'return_delay_time': 0,
        'stall_backoff': 0.017,
        'stall_max_effort': 10.0,
        'stall_max_time': 1.0,
        'stall_min_vel': 0.1,
        'temperature_limit': 72,
        'usb_name': '/dev/hello-dynamixel-wrist',
        'use_multiturn': 0,
        'zero_t': 2070,
        'baud': 115200,
        'retry_on_comm_failure': 1,
        'disable_torque_on_stop': 0
    },
    "wrist_roll": {
        'flip_encoder_polarity': 0,
        'enable_runstop': 1,
        'gr': 1.0,
        'id': 15,
        'max_voltage_limit': 15,
        'min_voltage_limit': 11,
        'motion': {
            'default': {'accel': 8.0, 'vel': 3.0},
            'fast': {'accel': 15.0, 'vel': 6.0},
            'max': {'accel': 15.0, 'vel': 6.0},
            'slow': {'accel': 4.0, 'vel': 1.0},
            'trajectory_max': {'accel_r': 16.0, 'vel_r': 8.0},
        },
        'pid': [640, 0, 0],
        'pwm_homing': [0, 0],
        'pwm_limit': 885,
        'range_t': [0, 4095],
        'req_calibration': 0,
        'return_delay_time': 0,
        'stall_backoff': 0.017,
        'stall_max_effort': 10.0,
        'stall_max_time': 1.0,
        'stall_min_vel': 0.1,
        'temperature_limit': 72,
        'usb_name': '/dev/hello-dynamixel-wrist',
        'use_multiturn': 0,
        'zero_t': 2090,
        'baud': 115200,
        'retry_on_comm_failure': 1,
        'disable_torque_on_stop': 0
    }}



from stretch_body.dynamixel_hello_XL430 import DynamixelHelloXL430
import stretch_body.hello_utils as hu


class WristRoll(DynamixelHelloXL430):
    """
    API to the Stretch RE1 wrist roll joint
    """
    def __init__(self, chain=None):
        DynamixelHelloXL430.__init__(self,'wrist_roll',chain)
        self.poses = {'cw_90': hu.deg_to_rad(90.0), 'forward': hu.deg_to_rad(0.0), 'ccw_90': hu.deg_to_rad(-90.0)}

    def home(self):
        """
        Home to hardstops
        """
        pass

    def pose(self,p,v_r=None,a_r=None):
        """
        p: Dictionary key to named pose (eg 'forward')
        v_r: velocityfor trapezoidal motion profile (rad/s).
        a_r: acceleration for trapezoidal motion profile (rad/s^2)
        """
        self.move_to(self.poses[p],v_r,a_r)

    def step_sentry(self,robot):
        """
        This sentry attempts to prevent the wrist servo from overheating
        if it is pushing against an object for too long
        It works by backing off the commanded position from the current position
        so as to lower the steady state error of the PID controller
        """
        DynamixelHelloXL430.step_sentry(self,robot)
        if self.robot_params['robot_sentry']['wrist_roll_overload']:
            if self.status['stall_overload']:
                if self.status['effort']>0:
                    self.move_by(self.params['stall_backoff'])
                    self.logger.debug('Backoff at stall overload')
                else:
                    self.move_by(-1*self.params['stall_backoff'])
                    self.logger.debug('Backoff at stall overload')



from stretch_body.dynamixel_hello_XL430 import DynamixelHelloXL430
import stretch_body.hello_utils as hu


class WristPitch(DynamixelHelloXL430):
    """
    API to the Stretch RE1 wrist pitch joint
    """
    def __init__(self, chain=None):
        DynamixelHelloXL430.__init__(self,'wrist_pitch',chain)
        self.poses = {'up': hu.deg_to_rad(90.0), 'forward': hu.deg_to_rad(0.0), 'down': hu.deg_to_rad(-90.0)}

    def home(self):
        """
        Home to hardstops
        """
        pass

    def pose(self,p,v_r=None,a_r=None):
        """
        p: Dictionary key to named pose (eg 'forward')
        v_r: velocityfor trapezoidal motion profile (rad/s).
        a_r: acceleration for trapezoidal motion profile (rad/s^2)
        """
        self.move_to(self.poses[p],v_r,a_r)





from stretch_body.end_of_arm_tools import EOA_Wrist_DW3_Tool_NIL


class EOA_Wrist_DW3_Tool_Tablet_12in(EOA_Wrist_DW3_Tool_NIL):
    def __init__(self, name='eoa_wrist_dw3_tool_tablet_12in'):
        EOA_Wrist_DW3_Tool_NIL.__init__(self, name)




from stretch_body.end_of_arm import EndOfArm


class ToolReactorWrist(EndOfArm):
    def __init__(self, name='tool_reactor_wrist'):
        EndOfArm.__init__(self, name)

    def stow(self):
        # Fold in wrist and gripper
        print('--------- Stowing ToolReactorWrist ----')
        self.move_to('wrist_pitch', self.params['stow']['wrist_pitch'])
        self.move_to('wrist_roll', self.params['stow']['wrist_roll'])
        self.move_to('wrist_yaw', self.params['stow']['wrist_yaw'])
        self.move_to('reactor_gripper', self.params['stow']['reactor_gripper'])

    def home(self):
        #self.motors['reactor_gripper'].home()
        self.motors['reactor_gripper'].move_to(0)
        self.motors['wrist_pitch'].move_to(0)
        self.motors['wrist_roll'].move_to(0)
        self.motors['wrist_yaw'].home()



import setuptools
from os import listdir
from os.path import isfile, join

script_path='./bin'
tool_scripts={script_path+'/'+f for f in listdir(script_path) if isfile(join(script_path, f))}

setuptools.setup(
    name="hello_robot_stretch_tool_share",
    version='0.3.3',
    author="Hello Robot Inc.",
    author_email="support@hello-robot.com",
    description="Stretch end of arm tool interfaces",
    url="https://github.com/hello-robot/stretch_tool_share",
    scripts = tool_scripts,
    packages=['stretch_tool_share',
              'stretch_tool_share_se3',
              'stretch_tool_share.reactorx_wrist_v1',
              'stretch_tool_share.usbcam_wrist_v1',
              'stretch_tool_share.dry_erase_holder_v1',
              'stretch_tool_share_se3.eoa_wrist_dw3_tool_tablet_12in'],
    classifiers=[
        "Programming Language :: Python :: 2",
        "Operating System :: OS Independent",
        "License :: OSI Approved :: GNU Lesser General Public License v3 (LGPLv3)"
    ]
)

